package kck.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.Timer;

import kck.HTDView;
import kck.logic.GeneralizedImage;

public class MainWindow extends JFrame {
	private static final long serialVersionUID = 1L;
	
	private GeneralizedImage gi;
	
	private JMenuBar menubar;
	private JMenu file;
	private JMenuItem file_open;
	private JMenuItem file_exit;
	
	private Drawing drawing;
	private JScrollPane drawingScroll;
	
	private JToolBar toolbar;
	private JComboBox t_view;
	private JComboBox cmap;
	private JTextField msd; // max seq dist
	private JTextField pd; // points dist
	private JTextField sa; // simplification angle
	private JTextField sr; // simplification radius
	private JButton anim; // animuj przebieg kreslenia
	private static String[] view_modes = {"Obraz", "Selekc.Pktw", "Siatka", "Zreduk.Siatka", "Figury"};
	private static String[] color_mappings = {"Nacisk", "Kt.nachylenia", "wysoko"};
	
	private int viewMode;
	private int maxSeqDist;
	private int pointsDist;
	private double simplAngle;
	private int simplRadius;
	
	private JFileChooser open_chooser;
	
	private Timer draw_timer;
	private int timestamp;
	private boolean drawingNow;
	
	private HTDView root;
	
	public MainWindow(HTDView root) {
		this.root = root;
		
		viewMode = 0;
		maxSeqDist = 5;
		pointsDist = 5;
		simplAngle = 0.3;
		simplRadius = 6;
		
		drawingNow = false;
	}
	
	public void initComponents() {
		menubar = new JMenuBar();
		file = new JMenu("Plik");
		file_open = new JMenuItem("Otwrz...");
		file_exit = new JMenuItem("Zakocz");

		open_chooser = new JFileChooser();
		
		toolbar = new JToolBar();
		
		toolbar.setPreferredSize(new Dimension(15, 22));
		toolbar.setMaximumSize(new Dimension(10000, 22));
		
		t_view = new JComboBox(view_modes);
		toolbar.add(t_view);
		toolbar.addSeparator();
		t_view.setPreferredSize(new Dimension(100, 22));
		t_view.setMaximumSize(new Dimension(100, 22));
		
		msd = new JTextField(Integer.toString(maxSeqDist));
		msd.setPreferredSize(new Dimension(40, 22));
		msd.setMaximumSize(new Dimension(40, 22));
		
		pd = new JTextField(Integer.toString(pointsDist));
		pd.setPreferredSize(new Dimension(40, 22));
		pd.setMaximumSize(new Dimension(40, 22));
		
		sa = new JTextField(Double.toString(simplAngle));
		sa.setPreferredSize(new Dimension(40, 22));
		sa.setMaximumSize(new Dimension(40, 22));
		
		sr = new JTextField(Integer.toString(simplRadius));
		sr.setPreferredSize(new Dimension(40, 22));
		sr.setMaximumSize(new Dimension(40, 22));
		
		cmap = new JComboBox(color_mappings);
		cmap.setPreferredSize(new Dimension(100, 22));
		cmap.setMaximumSize(new Dimension(100, 22));
		
		anim = new JButton("Przebieg");
		anim.setPreferredSize(new Dimension(80, 22));
		anim.setMaximumSize(new Dimension(80, 22));
		
		toolbar.add(new JLabel("MSD:"));
		toolbar.add(msd);
		toolbar.add(new JLabel("PD:"));
		toolbar.add(pd);
		toolbar.add(new JLabel("SA:"));
		toolbar.add(sa);
		toolbar.add(new JLabel("SR:"));
		toolbar.add(sr);
		toolbar.addSeparator();
		toolbar.add(new JLabel("Kolor:"));
		toolbar.add(cmap);
		toolbar.addSeparator();
		toolbar.add(anim);
		
		toolbar.add(Box.createHorizontalGlue());
		
		draw_timer = new Timer(10, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				timestamp += 10;
				
				if (! root.data.timedDraw(timestamp)) {
					draw_timer.stop();
				}
				drawing.repaint();
				
			}
		});
		
		t_view.setEnabled(false);
		msd.setEnabled(false);
		pd.setEnabled(false);
		sa.setEnabled(false);
		sr.setEnabled(false);
		cmap.setEnabled(false);
		anim.setEnabled(false);
		
		setJMenuBar(menubar);
		menubar.add(file);
		file.add(file_open);
		file.addSeparator();
		file.add(file_exit);
		
		drawing = new Drawing();
		
		drawingScroll = new JScrollPane(drawing);

		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		getContentPane().add(toolbar);
		getContentPane().add(drawingScroll);
	}
	
	private void makeDrawingVisible() {
		int reduction;
		drawing.clearLayers();
		
		if (viewMode == 0) {
			root.data.initDraw();
			drawing.addLayer(root.data);
		} else {
			gi = new GeneralizedImage(root.data);
			gi.stripClosePoints(pointsDist);
			gi.pointsDraw();
			
			if (viewMode >= 2) {
				gi.pointsToMesh1();

				if (viewMode >= 3) {
					do {
						reduction = gi.simplifyMesh(simplAngle, simplRadius);
					} while (reduction > 0);
				}
				
				gi.initDraw();
			}
			
			drawing.addLayer(gi);
		}
		
		drawing.repaint();
		drawingScroll.revalidate();
	}
	
	private void openDrawing() {
		if (open_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			
			try {
				root.data.setMaxSeqDist(maxSeqDist);
				root.data.load(open_chooser.getSelectedFile());
			} catch (IOException e) {
				
			};
			
			makeDrawingVisible();
			
			t_view.setEnabled(true);
			msd.setEnabled(true);
			pd.setEnabled(true);
			sa.setEnabled(true);
			sr.setEnabled(true);
			cmap.setEnabled(true);
			anim.setEnabled(true);
		}
	}
	
	public void initActions() {
		file_open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openDrawing();
			}
		});
		
		file_exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		
		msd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				maxSeqDist = Integer.parseInt(msd.getText());
				try {
					root.data.setMaxSeqDist(maxSeqDist);
					root.data.load(open_chooser.getSelectedFile());
				} catch (IOException ex) {};
				makeDrawingVisible();
			}
		});
		
		pd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pointsDist = Integer.parseInt(pd.getText());
				makeDrawingVisible();
			}
		});
		
		sa.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				simplAngle = Double.parseDouble(sa.getText());
				makeDrawingVisible();
			}
		});
		
		sr.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				simplRadius = Integer.parseInt(sr.getText());
				makeDrawingVisible();
			}
		});
		
		t_view.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				viewMode = t_view.getSelectedIndex();
				makeDrawingVisible();
			}
		});
		
		cmap.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				root.data.cs.setMode(cmap.getSelectedIndex());
				makeDrawingVisible();
			}
		});
		
		anim.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
				if (! drawingNow) {
					timestamp = 0;
					t_view.setSelectedIndex(0);
					root.data.timedDrawInit();
					draw_timer.start();
				} else {
					draw_timer.stop();
					makeDrawingVisible();
				}
				
				drawingNow = ! drawingNow;
			}
		});
	}

}
