package kck.logic;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.HashSet;
import java.util.Set;

public class GeneralizedImage extends HTDData {
	private Point matrix[][];
	private int count[][];
	private int matrixDensity;
	private int meshLines[][]; // trzyma numery punktow, miedzy ktorymi jest odcinek (linia siatki)
	private int meshLinesCount;
	
	public GeneralizedImage(HTDData source) {
		super(source);
	}

	public void toHTDData(HTDData dest) {
		dest.copyHeader(this);
		dest.copyBody(this);
	}

	/**
	* Reallocates an array with a new size, and copies the contents
	* of the old array to the new array.
	* @param oldArray  the old array, to be reallocated.
	* @param newSize   the new array size.
	* @return          A new array with the same contents.
	*/
	private static Object resizeArray(Object oldArray, int newSize) {
		int oldSize = java.lang.reflect.Array.getLength(oldArray);
		Class elementType = oldArray.getClass().getComponentType();
		Object newArray = java.lang.reflect.Array.newInstance(elementType,newSize);
		int preserveLength = Math.min(oldSize,newSize);
		if (preserveLength > 0) {
			System.arraycopy(oldArray,0,newArray,0,preserveLength);
		}
		return newArray; 
	}
	
	public void pointsToMesh1() {
		int xoffs[] = {1, 1, 0, -1};
		int yoffs[] = {0, 1, 1,  1};
		int dropx[] = {0, 0, 0,  0};
		int dropy[] = {-1, -1, -1, -1};
		
		pointsToMesh(xoffs, yoffs, dropx, dropy);
	}
	
	public void pointsToMesh2() {
		int xoffs[] = {2, 2, 1, 0, -1, -2};
		int yoffs[] = {0, 1, 2, 2,  2,  1};
		int dropx[] = {1, 1, 1, 0, -1, -1};
		int dropy[] = {0, 0, 1, 1,  1,  1};
		
		pointsToMesh(xoffs, yoffs, dropx, dropy);
	}
	
	public void pointsToMesh(int xoffs[], int yoffs[], int dropx[], int dropy[]) {
		int i;
		int ax;
		int ay;
		
		int bx;
		int by;
		
		int dx;
		int dy;
		
		int v;
		
		meshLines = new int[(int)(pointsCount*2)][2];
		meshLinesCount = 0;
		
		for (i=0; i<pointsCount; i++) {
			ax = x[i]/matrixDensity;
			ay = y[i]/matrixDensity;
			
			if (matrix[ax][ay] != null) { 
			
				for (v=0; v<xoffs.length; v++) {
					bx = ax + xoffs[v];
					by = ay + yoffs[v];
					
					if (matrix[bx][by] != null) {
						if (meshLinesCount == meshLines.length) {
							meshLines = (int[][])resizeArray(meshLines, (int)(meshLinesCount*1.5));
						}
						
						if (meshLines[meshLinesCount] == null) {
							meshLines[meshLinesCount] = new int[2];
						}
						
						if (dropy[v] != -1) {
							dx = ax + dropx[v];
							dy = ay + dropy[v];
							
							if (matrix[dx][dy] != null) {
								matrix[dx][dy] = null;
							}
						}
						
						
						int p = getPointAtMatrix(bx, by);
						if (p != -1) {
							meshLines[meshLinesCount][0] = i;
							meshLines[meshLinesCount][1] = p;
							meshLinesCount++;
						}
					}
				}
			}
		}
		
		matrixToPoints();
	}
	
	private int getPointAtMatrix(int px, int py) {
		int i=-1;
		int ax;
		int ay;
		
		do  {
			i++;
			ax = x[i]/matrixDensity;
			ay = y[i]/matrixDensity;
			if ((ax==px) && (ay==py)) {
				return i;
			}
		} while (i<pointsCount);
		return -1;
	}
	
	public int simplifyMesh(double angle, int radius) {
		int vertices[];
		int i;
		int reduction = meshLinesCount;
		
		vertices = new int[pointsCount];
		for (i=0; i<meshLinesCount; i++) {
			vertices[meshLines[i][0]]++;
			vertices[meshLines[i][1]]++;
		}
		
		for (i=0; i<pointsCount; i++) {
			if (vertices[i] == 2) {
				simplify3PointLine(vertices, i, angle);
			} 
			else if ((vertices[i] > 2) && (radius > 0)) {
				groupClique(i, radius);
			}
		}
		return reduction - meshLinesCount;
	}
	
	/**
	 * Dla wierzchoka i, szuka okolicznych wierzchokw, z ktrymi jest
	 * poczony i tych, z ktrymi one s poczone - w pewnym promieniu
	 * i wszystkie grupuje do jednego. w promieniu r.
	 * @param vertices
	 * @param i
	 */
	private void groupClique(int center, int radius) {
		HashSet toGather; // tablica punktow do zgrupowania 
		HashSet toConnect; // tablica punktow, do ktorych bedziemy laczyc
		HashSet linesToRemove; // tablica linii, ktore nalezy usunac
		
		toGather = new HashSet();
		toConnect = new HashSet();
		linesToRemove = new HashSet();
		
		findCloseNeighbour(toGather, toConnect, linesToRemove, center, center, radius);
		
		if (toGather.size() > 2) {
			//System.out.println("grouping with ("+center+") "+x[center]+"x"+y[center]+" toGather: "+toGather.size()+" toConnect: "+toConnect.size());
			removeConnections(linesToRemove);
			groupPoints(center, toGather);
			connectPoints(center, toConnect);
			//System.out.println("\tcenter: "+x[center]+"x"+y[center]);
		}
	}
	
	private void removeConnections(Set linesToRemove) {
		Object[] l = linesToRemove.toArray();
		int i;
		
		for (i=0; i<l.length; i++) {
			int n = ((Integer)l[i]).intValue();
			//System.out.println("\tremove "+x[meshLines[n][0]]+"x"+y[meshLines[n][0]]+"-"+x[meshLines[n][1]]+"x"+y[meshLines[n][1]]+" ("+meshLines[n][0]+"-"+meshLines[n][1]+") (meshline "+n+")");
			removeMeshLine(n);
		}
	}
	
	/**
	 * tworzy polaczenia z pointA do wszystkich punktow w zbiorze pointsB
	 * @param pointA
	 * @param pointsB
	 */
	private void connectPoints(int pointA, Set pointsB) {
		Object[] g = pointsB.toArray();
		int i;
		int n;
		
		for (i=0; i<g.length; i++) {
			n = ((Integer)g[i]).intValue();
			//System.out.println("\tconnect ("+pointA+"-"+n+") => (meshline "+(meshLinesCount)+")");
			meshLines[meshLinesCount][0] = pointA;
			meshLines[meshLinesCount][1] = n;
			meshLinesCount++;
		}
	}
	
	/**
	 * Grupuje punkty ze zbioru group i wstawia jako jeden punkt nr. destPointNo
	 * @param destPointNo
	 * @param group
	 */
	private void groupPoints(int destPointNo, Set group) {
		Object[] g = group.toArray();
		int i;
		int n;
		int dx = 0;
		int dy = 0;
		
		for (i=0; i<g.length; i++) {
			n = ((Integer)g[i]).intValue();
			//System.out.println("\tgroup "+x[n]+"x"+y[n]+" ("+n+")");
			dx += x[n];
			dy += y[n];
		}
		
		x[destPointNo] = dx/g.length;
		y[destPointNo] = dy/g.length;
	}
	
	/**
	 * 
	 * @param toGather
	 * @param toConnect
	 * @param i
	 * @param r
	 */
	private void findCloseNeighbour(HashSet toGather, HashSet toConnect, HashSet linesToRemove, int center, int current, int radius) {
		int i;
		int k;
		int dist; 
		
		for (i=0; i<meshLinesCount; i++) {
			
			if ((! linesToRemove.contains(new Integer(i))) && ((meshLines[i][0] == current) || (meshLines[i][1] == current))) {
				k = (meshLines[i][0] == current) ? meshLines[i][1] : meshLines[i][0];
				dist = (int)distance(center, k);
			
				linesToRemove.add(new Integer(i));
				
				if (dist < radius) {
					toGather.add(new Integer(k));
					findCloseNeighbour(toGather, toConnect, linesToRemove, center, k, radius);
				} else {
					toConnect.add(new Integer(k));
				}
			}
		}
	}
	
	/**
	 * Dla wierzchoka i, szuka dwch odcinkw, do ktrych on naley i czy
	 * je. z dwch odcinkw robi si jeden odcinek. punkt wsplny nie jest
	 * ju uywany (zakadamy, e by poczony tylko z tymi dwoma odcinkami)
	 * 
	 * @param vertices
	 * @param i
	 */
	private void simplify3PointLine(int vertices[], int i, double angle) {
		int j, to;
		int a;
		int b;
		
		a=-1;
		b=-1;
		
		j=0;
		while ((j<meshLinesCount) && (a == -1)) {
			a = ((meshLines[j][0] == i) || (meshLines[j][1] == i)) ? j : a;
			j++;
		}
		while ((j<meshLinesCount) && (b == -1)) {
			b = ((meshLines[j][0] == i) || (meshLines[j][1] == i)) ? j : b;
			j++;
		}
		
		if ((b != -1) && paralell(a, b, angle)) {
			to = (meshLines[a][0] == i) ? 0 : 1;
			meshLines[a][to] = (meshLines[b][0] != i) ? meshLines[b][0] : meshLines[b][1];
			
			removeMeshLine(b);
			
			vertices[i] = 0;
		}
	}
	
	private void removeMeshLine(int i) {
		meshLinesCount--;
		System.arraycopy(meshLines[meshLinesCount], 0, meshLines[i], 0, 2);
	}
	
	private boolean paralell(int i, int j, double difference) {
		int dxi = x[meshLines[i][0]] - x[meshLines[i][1]];
		int dyi = y[meshLines[i][0]] - y[meshLines[i][1]];
		int dxj = x[meshLines[j][0]] - x[meshLines[j][1]];
		int dyj = y[meshLines[j][0]] - y[meshLines[j][1]];
		
		if (Math.abs(dxi) <= Math.abs(dyi)) {
			return Math.abs((double)dxi/dyi - (double)dxj/dyj) < difference;
		} else {
			return Math.abs((double)dyi/dxi - (double)dyj/dxj) < difference;
		}
	}
	
	private void matrixToPoints() {
		int ax;
		int ay;
		
		pointsCount=0;
		
		for (ax=0; ax<matrix.length; ax++) {
			for (ay=0; ay<matrix[ax].length; ay++) {
				if (matrix[ax][ay] != null) {
					x[pointsCount] = matrix[ax][ay].x;
					y[pointsCount] = matrix[ax][ay].y;
					pressure[pointsCount] = HTDData.maxPressure/2;
					pointsCount++;
				}
			}
		}
	}
	
	public void pointsDraw() {
		super.initDraw();
	}
	
	public void initDraw() {
		dataDraw = new BufferedImage((int)(getMaxx() - getMinx()), (int)(getMaxy() - getMinx()), BufferedImage.TYPE_INT_ARGB);
		
		Graphics g;
		int i;
		
		g = dataDraw.getGraphics();
		
		//System.out.println(meshLinesCount);
		
		if (meshLinesCount > 0) {
			
			for (i=0; i<meshLinesCount; i++) {
				
				switch ((int)(Math.random()*4)) {
					case (0): g.setColor(Color.BLACK); break;
					case (1): g.setColor(Color.GREEN); break;
					case (2): g.setColor(Color.BLUE); break;
					case (3): g.setColor(Color.RED); break;
				}
				
				g.drawLine(x[meshLines[i][0]], 
						y[meshLines[i][0]], 
						x[meshLines[i][1]], 
						y[meshLines[i][1]]);
			}
		}
		
	}
	
	/**
	 * Usuwa ssiadujce ze sob si punkty, blisze ni dist pikseli.
	 * Na ich miejsce wstawia uredniony punkt.
	 * @param dist
	 */
	public void stripClosePoints(int dist) {
		int i;
		int ax;
		int ay;
		int w = (int)(maxx-minx)/dist;
		int h = (int)(maxy-miny)/dist;
		
		matrix = new Point[w][h];
		count = new int[w][h];
		matrixDensity = dist;
		
		for (i=0; i<pointsCount; i++) {
			ax = Math.min(w-1, x[i]/dist);
			ay = Math.min(h-1, y[i]/dist);
			
			count[ax][ay]++;
			
			if (count[ax][ay] == 1) {
				matrix[ax][ay] = new Point(x[i], y[i]);
			} else {
				matrix[ax][ay].x = (x[i] + (count[ax][ay]-1)*matrix[ax][ay].x)/count[ax][ay];
				matrix[ax][ay].y = (y[i] + (count[ax][ay]-1)*matrix[ax][ay].y)/count[ax][ay];
			}
		}

		matrixToPoints();
	}
	
}
