import java.util.ArrayList;

public class GameState {
	private ArrayList playersCards[];
	private ArrayList playersCardsOnTable[];
	private ArrayList playersAquiredCards[];
	
	public int getWinner()
	{
		int i=0;
		while ((i<getPlayersCount()) && (getPlayerCardsCount(i) < Card.colors.length*Card.values.length))
		{
			i++;
		}
		
		return (i<getPlayersCount()) ? i : -1;
	}
	
	public int cardsOnTableCount()
	{
		return playersCardsOnTable[0].size() + playersCardsOnTable[1].size();
	}
	
	public boolean isWar()
	{
		if ((cardsOnTableCount() == 0) || (cardsOnTableCount() % 2 ==1))
		{
			return false;
		}
		else
		{
			int cardNum = (int)Math.floor((playersCardsOnTable[0].size()-1)/2) * 2;
			Card LastCard1 = (Card)(playersCardsOnTable[0].get(cardNum));
			Card LastCard2 = (Card)(playersCardsOnTable[1].get(cardNum));
			return ((LastCard1 != null) && (LastCard2 != null) && (LastCard1.equalsByValue(LastCard2)));
		}
	}
	
	public void reloadCards(int playerNo)
	{
		ArrayList a = playersCards[playerNo];
		playersCards[playerNo] = playersAquiredCards[playerNo];
		playersAquiredCards[playerNo] = a;
	}
	
	public void giveCardsOnTableTo(int playerNo)
	{
		int i;
		for (i=0; i<playersCardsOnTable.length; i++)
		{
			playersAquiredCards[playerNo].addAll(playersCardsOnTable[i]);
			playersCardsOnTable[i].clear();
		}
	}
	
	public int hasBetterCard()
	{
		return ((Card)playersCardsOnTable[0].get(playersCardsOnTable[0].size()-1)).getValue() > ((Card)playersCardsOnTable[1].get(playersCardsOnTable[1].size()-1)).getValue() ? 0 : 1;
	}
	
	public int getPlayerCardsCount(int playerNo)
	{
		return playersCards[playerNo].size() + playersAquiredCards[playerNo].size();
	}
	
	public void putCard(int playerNo)
	{
		if (getPlayerCardsCount(playerNo) > 0)
		{
			if (getPlayerCards(playerNo).size() == 0)
			{
				reloadCards(playerNo);
			}
			playersCardsOnTable[playerNo].add(Cards.getRandomCards(playersCards[playerNo], 1).get(0));
		}
	}
	
	public int getPlayersCount()
	{
		return playersCards.length;
	}
	
	public ArrayList getPlayerCards(int playerNo)
	{
		return new ArrayList(playersCards[playerNo]);
	}
	
	public ArrayList getPlayerAquiredCards(int playerNo)
	{
		return new ArrayList(playersAquiredCards[playerNo]);
	}
	
	/**
	 * Returns cards on table put by player with number specified
	 * by argument
	 * @param playerNo
	 * @return
	 */
	public ArrayList getPlayerCardsOnTable(int playerNo)
	{
		return new ArrayList(playersCardsOnTable[playerNo]);
	}
	
	/**
	 * Sets up new game
	 * @param playersCount
	 */
	public GameState(int playersCount)
	{
		ArrayList allCards = Cards.getAllCards();
		int cardsPerPlayer = allCards.size()/playersCount;
		int i;
		
		playersCards = new ArrayList[playersCount];
		playersCardsOnTable = new ArrayList[playersCount];
		playersAquiredCards = new ArrayList[playersCount];
		
		for (i=0; i<playersCount; i++)
		{
			playersCards[i] = new ArrayList(Cards.getRandomCards(allCards, cardsPerPlayer));
			playersCardsOnTable[i] = new ArrayList();
			playersAquiredCards[i] = new ArrayList();
		}
	}
}
