package src.visual;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import src.logical.Funkcja;
import src.logical.Profil;

public class VisualizationPanel extends JPanel {
	public VisualizationPanel(Main_Frame rodzic, int Paleta){
		super();
		oknoGlowne=rodzic;
		this.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		this.addMouseListener(new MyMouseListener());
		this.addMouseMotionListener(new MyMouseMotionListener());
		this.addMouseWheelListener(new MyMouseWheelListener());
		palette=Paleta;
		//this.setToolTipText("TT");
		
	}
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		drawProfile();
	}
	
	public void init(){
		BufImg = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_INT_RGB);
		BufImgWhite = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_INT_RGB);
		BufImgScreen = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_INT_RGB);
		BufImgWhite.getGraphics().fillRect(0,0,this.getWidth(),this.getHeight());
	}
	
	public void draw(){
		offsetX=0;
		offsetY=0;
		magnify=0;
		correctScale();
		oknoGlowne.setZoomSlider(magnify,maxMagnify);
		drawProfile();
	}
	
	private void getProfileImage(BufferedImage BImg){
		Color col;
		for(int i=0;i<BImg.getWidth();i++)
			for(int j=0;j<BImg.getHeight();j++){
				col = getColor(funkcja.f(((i)/scale+ProfileRectangle.x),
						(j)/scale+ProfileRectangle.y));
				BImg.setRGB(i,j,col.getRGB());
			}
	}
	
	
	private void drawProfile(){
		if((profil!=null)&&(funkcja!=null)){
			if(changes){
				getProfileImage(BufImg);
				changes=false;
			}
			
		BufImgScreen.getGraphics().drawImage(BufImgWhite,0,0,this);
		BufImgScreen.getGraphics().drawImage(BufImg,
				offsetX,offsetY,
				(int)(this.getWidth()*scale/baseScale)+offsetX,
				(int)(this.getHeight()*scale/baseScale)+offsetY,
				0,0,
				BufImg.getWidth(),
				BufImg.getHeight(),
				this);
		
		Graphics g =this.getGraphics();
		g.setClip(1,1,this.getWidth()-3,this.getHeight()-3);
		g.drawImage(BufImgScreen,0,0,this);
		if(f>-1)drawValueFrame((Graphics2D)g);//rysuje wartosc funkcji (tylko jak wewnatrz figury
		g.dispose();
		}
	}
	
	public void setProfile(Profil p){
		profil=p;
		ProfileRectangle=p.getRectangle();
		
		float DeltaX=ProfileRectangle.width;
		float DeltaY=ProfileRectangle.height;
		
		if((DeltaX/this.getWidth())>(DeltaY/this.getHeight()))
			scale=this.getWidth()/DeltaX;
		else
			scale=this.getHeight()/DeltaY;
		
		baseScale=scale;
		
	}
	
	public void setFunction(Funkcja f){
		funkcja=f;
	}
	
	/**
	 * ustawia nowe powiekszenie i uaktualnia podgad
	 * @param newMagnify (0..1)
	 */
	public void setMagnify(float newMagnify){
		magnify=(int)(maxMagnify*newMagnify);
		correctScale();
		drawProfile();
	}
	
	public void setPalette(int p){
		//this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		palette=p;
		changes=true;
		drawProfile();
		//this.setCursor(Cursor.getDefaultCursor());
	}
	
	private double maxf = 1; // maksymalna znana wartosc f(x,y)
	
	/**
	 * 
	 * @param value wartosc funkcji naprezenia w przedziale od 0-min do 1000-max
	 * @return kolor
	 */
	private Color getColor(float val){
	int value=(int)val;
	
	if (value>maxf) {
		maxf = value;
		changes = true;
	}
	
	if(palette==P_RGB){
	if(value<0)return (new Color(255,255,255));else	
		if((value/maxf)<0.5) return(new Color((int)((double)(value/maxf)*2*250),0,250));
		else return(new Color(255,0,(int)((double)2*(1.0 - value/maxf)*250)));
	}
	else
	{	if(value<0)return (new Color(255,255,255));else
	{
		int cval = (((value/maxf)*100) % 2) <= 0.2 ? (int)((double)(value/maxf)*255) : 200;
		return(new Color(cval, cval, cval));
	}
	}
	}
	
	
	private void correctScale(){
		//if(magnify>=0)scale=baseScale*(1+magnify/10F);
		//else
		//changes=true;
		double oldScale=scale;
		if(magnify>0)
		scale=baseScale*(1+magnify_step_size*magnify/(float)(maxMagnify+1));
		else
		{scale=baseScale*(1+magnify_step_size*magnify/(float)(magnify_step_size*maxMagnify+1));
		if(scale<0)System.out.println("scale<0 magnify="+magnify);
		}
		//BufImg = new BufferedImage((int)(this.getWidth()*scale/baseScale),(int)(this.getHeight()*scale/baseScale),BufferedImage.TYPE_INT_RGB);
		
		
		//int c = (int) (oldScale>scale?magnify_step_size:-magnify_step_size);
		//procent niepowiekszonego obrazka w ktorym jest myszka
		double px = (oldMouseX-offsetX)/(this.getWidth()*oldScale/baseScale);
		double py = (oldMouseY-offsetY)/(this.getHeight()*oldScale/baseScale);
		//przyrownuje aby stary i nowy procent polozenia (wszystko stale oprocz skali i nowego offsetu) i wyliczam nowy offset
		offsetX=(int) (-(px*(this.getWidth()*scale/baseScale))+oldMouseX);
		offsetY= (int) (-(py*(this.getHeight()*scale/baseScale))+oldMouseY);
		System.out.println("px, py: "+px+",  "+py);
		if(LastReDrawThread!=null){
			LastReDrawThread.interrupt();
		}
		else
		{LastReDrawThread= new ReDrawThredClass();
		 LastReDrawThread.start();
		}

		//ReDrawThredClass ReDrawThred = new ReDrawThredClass();
		//LastReDrawThread = ReDrawThred;
		//ReDrawThred.start();
	}
	
	private void drawValueFrame(Graphics2D g2d){
		g2d.setComposite(makeComposite(0.5F));
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		RoundRectangle2D ramka = new RoundRectangle2D.Double(
				//this.getWidth()-27,this.getHeight()-22,25,20,10,10);
				oldMouseX+2,oldMouseY+2,25,20,10,10);
		g2d.setPaint(new Color(50,50,100));
		g2d.setStroke(new BasicStroke(2));
		g2d.fill(ramka);
		g2d.setComposite(makeComposite(1));
		g2d.setPaint(new Color(250,250,100));
		g2d.drawString(""+f,//this.getWidth()-22,this.getHeight()-8);
				oldMouseX+7,oldMouseY+16);
	}
	
	
	/**Tworzy compozyt przezroczystoci
	 * @param alpha przezroczystosc
	 * @return wygenerowany AlphaComposite
	 */
	public static AlphaComposite makeComposite(float alpha) {
	    int type = AlphaComposite.SRC_OVER;
	    return(AlphaComposite.getInstance(type, alpha));
	  }
	
	private class MyMouseMotionListener implements MouseMotionListener {

		public void mouseDragged(MouseEvent arg0) {
			int newX = arg0.getX();
			int newY = arg0.getY();
			offsetX += (newX - oldMouseX);
			offsetY += (newY - oldMouseY);
			oldMouseX=newX;
			oldMouseY=newY;
			
			drawProfile();
		}

		public void mouseMoved(MouseEvent arg0) {
			if(funkcja!=null)
			f = (int) funkcja.f(((arg0.getX()-offsetX)/scale+ProfileRectangle.x),
					(arg0.getY()-offsetY)/scale+ProfileRectangle.y);
			oldMouseX=arg0.getX();
			oldMouseY=arg0.getY();
			drawProfile();
		}
		
	}
	
	private class MyMouseListener implements MouseListener {

		public void mouseClicked(MouseEvent arg0) {}
		public void mousePressed(MouseEvent arg0) {
			oldMouseX=arg0.getX();
			oldMouseY=arg0.getY();
		}
		public void mouseReleased(MouseEvent arg0) {}
		public void mouseEntered(MouseEvent arg0) {}
		public void mouseExited(MouseEvent arg0) {}

	}
	
	public class MyMouseWheelListener implements MouseWheelListener {

		public void mouseWheelMoved(MouseWheelEvent arg0) {
			magnify-=arg0.getUnitsToScroll();
			if(magnify>maxMagnify)magnify=maxMagnify;else
			if(magnify<minMagnify)magnify=minMagnify;else{
				correctScale();
				oknoGlowne.setZoomSlider(magnify,maxMagnify);
				//scale=((100+3*arg0.getUnitsToScroll())*scale)/100;
				drawProfile();
			}
		}

	}
	
	
	
	
	private static final long serialVersionUID = 1L;
	private BufferedImage BufImg;
	private BufferedImage BufImgWhite;
	public BufferedImage BufImgScreen;
	private Rectangle ProfileRectangle;
	private int palette=P_RGB;
	
	
	private double baseScale; //pamieta wlasciwa skale
	private double scale;
	private Profil profil;
	private Funkcja funkcja;
	private int offsetX=0;
	private int offsetY=0;
	private int magnify=0;
	private final int maxMagnify=100; //jaka jest rozdzielczosc powiekszania = ile jest krokow
	private final int minMagnify=-100;
	private float magnify_step_size=3; //jaki jest krok powiekszenia
	
	private int oldMouseX;
	private int oldMouseY;
	public boolean changes=true; //czy sa zmaiany i trzeba od nowa odczytywac profil
	private int f=-1; //wartosc funkcji w punkcie myszki
	
	Main_Frame oknoGlowne;
	public static final int P_RGB = 1;
	public static final int P_BW = 0;
	
	private ReDrawThredClass  LastReDrawThread;
	//private static int count=0;

class ReDrawThredClass extends Thread{
	
	public void run(){
		System.out.println("ruszam watek");
		redraw=true;
			do{
				if(Thread.currentThread().interrupted())redraw=true;
				while(redraw)redraw=WczytajObraz();
				try {
					sleep(1000);
				} catch (InterruptedException e) {
					//System.out.println("obudzony");
					redraw=true;
					//e.printStackTrace();
				}
			}
			while(true);
		}
	
	public boolean WczytajObraz(){
		TmpBufImg = new BufferedImage(
				(int)(VisualizationPanel.this.getWidth()*scale/baseScale),
				(int)(VisualizationPanel.this.getHeight()*scale/baseScale),
				BufferedImage.TYPE_INT_RGB);
		
		Color col;
		for(int i=0;i<TmpBufImg.getWidth();i++){
			if(Thread.currentThread().interrupted()){
				//System.out.println(/*count+*/" Przerwanie watku");
				TmpBufImg=null;
				//count++;
				redraw=true;
				return true;
			}
			for(int j=0;j<TmpBufImg.getHeight();j++){
				col = getColor(funkcja.f(((i)/scale+ProfileRectangle.x),
						(j)/scale+ProfileRectangle.y));
				TmpBufImg.setRGB(i,j,col.getRGB());
			}
		}
		BufImg=TmpBufImg;
		drawProfile();
		return false;

	}
	private BufferedImage TmpBufImg;
	boolean redraw;

}
}